@@ -1,6 +1,6 @@
 # Traffic Monitor Detector Raw Data (TMD_I480)
 # modify from TMD
-#
+# v6,v7,v8,v9
 #
 # parsing Traffic Monitor Detector
 # hardware:(Batman-201/601): ISK IWR6843/1843
@@ -9,8 +9,16 @@
 # ver:0.1.1 2021/09/16
 # ver:2.0.1 2022/04/28
 # ver:2.0.2 2022/05/12
+# ver:2.0.3 2022/05/12
+# ver:2.0.4 2022/05/15
 #===========================================
 # output: V6,V7,V8,v9 Raw data
+#
+# playback data type:
+# 	v6: list/dataFrame
+# 	v7: list/dataFrame
+# 	v8: dictionary
+# 	v9: dictionary
 
 
 import serial
@@ -18,6 +26,7 @@ import time
 import struct
 import pandas as pd
 import numpy as np
+import csv
 
 class header:
 	version = 0
@@ -53,14 +62,14 @@ class TrafficMD_I480:
 	v6_col_names = ['time','fN','type','range','azimuth','elv' ,'doppler','sx', 'sy', 'sz']
 	v7_col_names = ['time','fN','type','posX','posY','posZ','velX','velY','velZ','accX','accY','accZ','tid']
 	v8_col_names = ['time','fN','type','targetID']
-	v9_col_names = ['fN','type','snr','noise']
+	v9_col_names = ['time','fN','type','snr','noise']
 	sim_startFN = 0
 	sim_stopFN  = 0 
 	
 	v6simo = []
 	v7simo = []
-	v8simo = []
-	v9simo = []
+	v8simo = {}
+	v9simo = {}
 	
 	# add for interal use
 	tlvLength = 0
@@ -499,8 +508,8 @@ class TrafficMD_I480:
 					
 					#print("V7:dataBytes({:d}) lenCount({:d}) index:{:d}".format(dataBytes,lenCount,idx))
 					try:
+						
 						(tid,posX,posY,posZ,velX,velY,velZ,accX,accY,accZ,ec0,ec1,ec2,ec3,ec4,ec5,ec6,ec7,ec8,ec9,ec10,ec11,ec12,ec13,ec14,ec15,g,conf) = struct.unpack('I27f', sbuf) 
-						#fieldNames = ['time','frameNum','type','range/px','azimuth/py','elv/vx','doppler/vy','sx/accX','sy/accY','sz/pz','na/vz','na/accZ','na/ID']
 						if (df == 'DataFrame' ):
 							v7df.append((self.hdr.frameNumber,'v7',posX,posY,posZ,velX,velY,velZ,accX,accY,accZ,tid))
 						else:
@@ -563,71 +572,55 @@ class TrafficMD_I480:
 					#return (False,v6,v7,v8,v9)
 					return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
 
-	def getRecordData(self,frameNum):
-		s_fn = frameNum + self.sim_startFN
-		#print("frame number:{:}".format(s_fn))
-		v6d = self.v6simo[self.v6simo['fN'] == s_fn]
-		#v6d =  v6dd[v6dd['doppler'] < 0.0]
-		#print("==============v6d lib==================")
-		v7d = self.v7simo[self.v7simo['fN'] == s_fn]
-		v8d = self.v8simo[self.v8simo['fN'] == s_fn]
-		v9d = self.v9simo[self.v9simo['fN'] == s_fn]
-		chk = 0
-		if v6d.count != 0:
-			chk = 1
-		return (chk,v6d,v7d,v8d,v9d)
-		
-		
-	'''
-	v6_col_names_rt = ['fN','type','range','azimuth','elv' ,'doppler','sx', 'sy', 'sz'] #ok
-	v7_col_names_rt = ['fN','type','posX','posY','velX','velY','accX','accY','posZ','velZ','accZ','tid']
-	v8_col_names_rt = ['fN','type','targetID']
-	v9_col_names_rt = ['fN','type','snr','noise']
 	
-	# read from file for trace point clouds
-	fileName = ''
-	v6_col_names = ['time','fN','type','range','azimuth','elv' ,'doppler','sx', 'sy', 'sz']
-	v7_col_names = ['time','fN','type','posX','posY','velX','velY','accX','accY','posZ','velZ','accZ','tid']
-	v8_col_names = ['time','fN','type','targetID']
-	v9_col_names = ['fN','type','snr','noise']
-	'''
+	############## for playback use #####################
 	def readFile(self,fileName):
-		#fileName = "pc32021-03-19-10-02-17.csv"  
-		#df = pd.read_csv(fileName, error_bad_lines=False, warn_bad_lines=False) 
-		self.fileName = fileName 
-		#          ['time','fN','type','elv','azimuth','range' ,'doppler','sx', 'sy', 'sz']
-		df = pd.read_csv(self.fileName, names = self.v6_col_names, skiprows = [0,10,11,12],dtype={'fN': int,'elv': float,'azimuth':float,'range':float,'doppler':float,'sx':float,'sy':float,'sz':float}) 
-		df.dropna()
-		#print("------------------- df --------------------shape:{:}".format(df.shape))
-		print(df)
-		print(df.info())
-		#print(df.info(memory_usage="deep")) 
-		
-		v6simOri = df[(df.type == 'v6')]
-		#print("-------------------v6simo------------:{:}".format(v6simOri.shape))
-									 
-		self.v6simo = v6simOri.loc[:,['fN','type','range','azimuth','elv' ,'doppler','sx', 'sy', 'sz']]
+		with open(fileName, newline='') as csvfile:
+			rows = csv.reader(csvfile)
+			v6s = [];v7s = [];v8s = [];v9s = []
+			for row in rows:
+				if row[2] == 'v6':
+					v6s.append(row)
+				elif row[2] == 'v7':
+					v7s.append(row)
+				elif row[2] == 'v8':
+					v8s.append(row)
+				elif row[2] == 'v9':
+					v9s.append(row)
+					
+		v6_col_names = ['time','fN','type','range','azimuth','elv' ,'doppler','sx', 'sy', 'sz']
+		self.v6simo = pd.DataFrame(v6s,columns = v6_col_names).astype({'fN': 'int32','type':str,'range':float,'azimuth':float,'elv': float,'doppler':float,'sx':float,'sy':float,'sz':float},errors = 'raise') 
 		
 		if len(self.v6simo):
-			self.sim_startFN = df['fN'].values[0]
-			self.sim_stopFN  = df['fN'].values[-1]
-		#print(self.v6simo)
+			#print(f"self.v6simo['fN'].values[0] = {self.v6simo['fN'].values[0]}")
+			self.sim_startFN =  self.v6simo['fN'].values[0]
+			self.sim_stopFN  =  self.v6simo['fN'].values[-1]
+			
+		v7_col_names = ['time','fN','type','posX','posY','posZ','velX','velY','velZ','accX','accY','accZ','tid'] 
+		self.v7simo = pd.DataFrame(v7s,columns = v7_col_names )
 		
-		self.v6simo['elv'] = self.v6simo['elv'].astype(float, errors = 'raise') 
+		for item in v9s:
+			hh = item[3].strip('[]').replace('(', "").replace(')','').split(',')
+			self.v9simo[int(item[1])] = [(int(hh[i]),int(hh[i+1])) for i in range(0,len(hh),2)]
 		
-		df7 = pd.read_csv(self.fileName, names = self.v7_col_names, skiprows = [0])  
-		
-		v7simc = df7[df7['type'] == 'v7']
-		self.v7simo  = v7simc.loc[:,['fN','type','posX','posY','velX','velY','accX','accY','posZ','velZ','accZ','tid']]
-		
-		v8simc = df[df['type'] == 'v8']
-		self.v8simo  = v8simc.loc[:,['fN','elv']]
-		#print(self.v8simo)
-		
-		v9simc = df[df['type'] == 'v9']
-		self.v9simo  = v9simc.loc[:,['fN','type','range','azimuth']]
-		self.v9simo.columns = ['fN','type','snr','noise']
+		for item in v8s:
+			hh = item[3].strip('[]').split(',')
+			self.v8simo[int(item[1])] = [int(hh[i]) for i in range(len(hh))] 
+			
+
+	def getRecordData(self,frameNum):
+		s_fn = frameNum + self.sim_startFN
+		v6d = self.v6simo.loc[self.v6simo['fN'] == s_fn]  #if (len(self.v6simo) > 0) else []
+		v7d = self.v7simo.loc[self.v7simo['fN'] == s_fn]  #if (len(self.v7simo) > 0) else []
+		try:
+			v8d =  self.v8simo[s_fn]    
+		except:
+			v8d = {}
+			
+		try:
+			v9d =  self.v9simo[s_fn] 
+		except:
+			v9d = {}
+		 
+		return(1,v6d,v7d,v8d,v9d)
 	
-		return (self.v6simo,self.v7simo,self.v8simo,self.v9simo)
-
-
